#ifndef __AT_CMD_H
#define __AT_CMD_H	 
#include <at32wb415.h>

typedef enum
{
  AT_CMD_WAIT_REQ                        = 0x0,
  AT_CMD_WAIT_RSP                        = 0x1
} at_cmd_state_machine;

typedef enum 
{
  RSP_ERROR                              = 0x0,
  RSP_OK                                 = 0x1,
  RSP_WAIT                               = 0x2
} cmd_rsp_status;

typedef enum 
{
  REGION_FLASH                           = 0x0,
  REGION_SRAM                            = 0x1
} cmd_region;

typedef enum 
{
  BAUD_RATE_9600                         = 0x0,
  BAUD_RATE_19200                        = 0x1,
  BAUD_RATE_38400                        = 0x2,
  BAUD_RATE_57600                        = 0x3,
  BAUD_RATE_115200                       = 0x4
} cmd_baud_rate;

typedef enum 
{
  INTERVAL_100MS                         = 0x0,
  INTERVAL_250MS                         = 0x1,
  INTERVAL_500MS                         = 0x2,
  INTERVAL_1600MS                        = 0x3,
  INTERVAL_3200MS                        = 0x4
} cmd_adv_interval;

typedef enum 
{
  FLASH_IDLE                             = 0x0,
  FLASH_READING                          = 0x1
} cmd_flash_read_status;

struct at_cmd_msg
{
  char* at_cmd_string;
  uint8_t msg_id;
};

#define BD_NAME_LENGTH                   (64)

// Request CMD
#define AT_CMD_BAUDS_9600                "AT+BAUDS1\r\n"
#define AT_CMD_BAUDS_19200               "AT+BAUDS2\r\n"
#define AT_CMD_BAUDS_38400               "AT+BAUDS3\r\n"
#define AT_CMD_BAUDS_57600               "AT+BAUDS4\r\n"
#define AT_CMD_BAUDS_115200              "AT+BAUDS5\r\n"

#define AT_CMD_VOLUME_DOWN               "AT+VOL0\r\n"
#define AT_CMD_VOLUME_UP                 "AT+VOL1\r\n"
#define AT_CMD_WWW_BACK                  "AT+WWWBACK\r\n"
#define AT_CMD_MOUSE_VALUE               "AT+X127Y127W0B0\r\n"

//Received Message ID
#define AT_RSP_OK                        "OK"
#define AT_RSP_ERROR                     "ERROR"

#define AT_RST_VOL_OK0                   "OKVOL0\r\n"
#define AT_RST_VOL_OK1                   "OKVOL1\r\n"
#define AT_RST_WWW_BACK_OK               "OKWWWBACK\r\n"
#define AT_RST_MOUSE_OK                  "OKMOUSE\r\n"

void at_cmd_send(char* string);
cmd_rsp_status cmd_rsp_get(char* string);
void at_cmd_bd_name_set(cmd_region mem, char* bd_name);
void at_cmd_baud_rate_set(cmd_region mem, cmd_baud_rate baud_rate);
void at_cmd_service_uuid_set(cmd_region mem, uint16_t uuid);
void at_cmd_chatateristic_write_uuid_set(cmd_region mem, uint16_t uuid);
void at_cmd_chatateristic_notify_uuid_set(cmd_region mem, uint16_t uuid);
uint8_t at_cmd_data_read(uint8_t addr);
void at_cmd_data_write(uint8_t addr, uint8_t data);
uint8_t flash_data_rsp_to_hex(char* rsp);
#endif
